library view_model;

import 'package:flutter/widgets.dart';

class ViewModelProvider extends InheritedWidget {
  final ViewModelFactory factory;

  ViewModelProvider({this.factory, Widget child}) : super(child: child);

  @override
  bool updateShouldNotify(ViewModelProvider oldWidget) =>
      factory != oldWidget.factory;

  static final Map<Type, ViewModel> _cache = <Type, ViewModel>{};

  T provide<T extends ViewModel>(Type type) {
    if (_cache.containsKey(type)) {
      return _cache[type];
    } else {
      final viewModel = factory.crateViewModel<T>(type, this);
      _cache[type] = viewModel;
      return viewModel;
    }
  }


  static ViewModelProvider of(BuildContext context) =>
      context.inheritFromWidgetOfExactType(ViewModelProvider);

  void forgetViewModel(Type type) {
    _cache.remove(type);
  }
}

abstract class ViewModelFactory {
  T crateViewModel<T extends ViewModel>(Type type, ViewModelProvider provider);
}

class ViewModel extends Disposed {
  ViewModelProvider _provider;


  ViewModel(this._provider);

  static T of<T extends ViewModel>(BuildContext context) {
    var tmp = _InheritedViewModel<T>();
    Type type = tmp.runtimeType;
//    Type type = _InheritedViewModel<T>; TODO: should be so
    var iVM = context.inheritFromWidgetOfExactType(type) as _InheritedViewModel<T>;
    return iVM.viewModel;
  }

  static Widget newInheritedWidget<T extends ViewModel>(
      {@required T viewModel, Widget child}) =>
      _InheritedViewModel<T>(viewModel: viewModel, child: child);

  @override
  void dispose() {
    _provider.forgetViewModel(this.runtimeType);
  }
}

abstract class Disposed {
  void dispose() {}
}

class _InheritedViewModel<T extends ViewModel> extends InheritedWidget {
  final T viewModel;

  _InheritedViewModel({this.viewModel, Widget child}) : super(child: child);

  @override
  bool updateShouldNotify(_InheritedViewModel oldWidget) =>
      viewModel != oldWidget.viewModel;

}
